import { UIOpacity } from 'cc';
import { _decorator, CCInteger, Component, EPhysics2DDrawFlags, EventTouch, Graphics, HingeJoint2D, Input, instantiate, Node, PhysicsSystem2D, Prefab, RigidBody2D, UITransform, Vec2, Vec3, VerticalTextAlignment } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('ShowerHead')
/**
 * 花洒效果
 */
export class ShowerHead extends Component {

    //固定处
    @property(Node)
    private Fix: Node | null = null;

    //头部
    @property(Node)
    private Head: Node | null = null;

    //关节数量
    @property(CCInteger)
    private JointNum: number = 0;

    //关节预设体
    @property(Prefab)
    private Joint: Prefab | null = null;

    //关节数组
    private JointArray: Node[] = [];

    //绳子绘制
    @property(Graphics)
    private RopeGra: Graphics | null = null;

    //绳子预设体
    @property(Prefab)
    private Rope: Prefab | null = null;
    private RopeArray: Node[] = [];

    //绳子层
    @property(Node)
    private RopeLoad: Node | null = null;

    //水滴层
    @property(Node)
    private WaterLoad: Node | null = null;

    //水滴预设体
    @property(Prefab)
    private Water: Prefab | null = null;

    //手指是否点击
    private IsClick: boolean = false;

    //手指滑动速度值
    private MoveScale: Vec2 = new Vec2(2, 2);

    //创建间隔
    private CreateTime: number = 0;

    start() {

        //debug测试
        // PhysicsSystem2D.instance.debugDrawFlags = EPhysics2DDrawFlags.Aabb |
        //     EPhysics2DDrawFlags.Pair |
        //     EPhysics2DDrawFlags.CenterOfMass |
        //     EPhysics2DDrawFlags.Joint |
        //     EPhysics2DDrawFlags.Shape;

        //创建更新关节
        for (let i = 0; i < this.JointNum; i++) {
            let joint: Node = instantiate(this.Joint);
            this.node.addChild(joint);
            this.JointArray.push(joint);
        }

        this.JointArray.push(this.Head);
        this.JointArray.unshift(this.Fix)


        for (let i = 0; i < this.JointArray.length - 1; i++) {
            this.UpdateJoint(this.JointArray[i], this.JointArray[i + 1]);
        }

        //添加绳子
        for (let i = 0; i < this.JointArray.length; i++) {
            let rope: Node = instantiate(this.Rope);
            this.RopeLoad.addChild(rope);
            rope.setSiblingIndex(0)
            this.RopeArray.push(rope);
        }

        //添加触摸事件
        this.Head.on(Input.EventType.TOUCH_START, () => {
            this.IsClick = true;
        }, this);

        this.node.on(Input.EventType.TOUCH_MOVE, (_e: EventTouch) => {
            if (!this.IsClick) return;
            let angle_num: number = this.getPointAngle(new Vec2(0, 0), _e.getDelta())
            this.Head.getComponent(RigidBody2D).linearVelocity = _e.getDelta().multiply(this.MoveScale)
        }, this);

        this.node.on(Input.EventType.TOUCH_END, this.TouchEnd, this);
        this.node.on(Input.EventType.TOUCH_CANCEL, this.TouchEnd, this);

    }

    /**
     * 更新关节链接属性
     */
    private UpdateJoint(_node1: Node, _node2: Node) {
        _node1.getComponent(HingeJoint2D).connectedBody = _node2.getComponent(RigidBody2D);
    }

    /**
     * 手指抬起事件
     */
    private TouchEnd() {
        this.IsClick = false;
    }

    update(deltaTime: number) {

        //更新绘制
        this.RopeGra.clear();
        this.RopeGra.moveTo(this.Fix.getPosition().x, this.Fix.getPosition().y);
        for (let i = 1; i < this.JointArray.length; i++) {
            this.RopeGra.lineTo(this.JointArray[i].getPosition().x, this.JointArray[i].getPosition().y)
        }
        // this.RopeGra.lineTo(this.Head.getPosition().x, this.Head.getPosition().y)
        this.RopeGra.stroke();

        //更新纹理
        for (let i = 1; i < this.JointArray.length; i++) {
            this.RopeArray[i].setPosition(this.JointArray[i - 1].getPosition());
            this.RopeArray[i].getComponent(UITransform).height = this.getPointDistance(this.JointArray[i].getPosition(), this.JointArray[i - 1].getPosition()) + 8
            this.RopeArray[i].setRotation(this.JointArray[i].getRotation())
        }
        this.RopeArray[0].active = false;

        //创建水滴
        if (!this.IsClick) return;

        this.CreateTime++;
        if (this.CreateTime % 5 == 0) {
            let angle: number = (this.Head.eulerAngles.z - 90) * Math.PI / 180;
            let angle1: number = -3.14 + Math.random() * 3.14;
            let water: Node = instantiate(this.Water);
            this.WaterLoad.addChild(water);
            water.setPosition(this.Head.getPosition().add(new Vec3(40 * Math.cos(angle1), 20 * Math.sin(angle1), 0)))
            water.getComponent(RigidBody2D).linearVelocity = new Vec2(20 * Math.cos(angle), 20 * Math.sin(angle));
            water.angle = angle * 180 / Math.PI - 90;
            water.getComponent(UIOpacity).opacity = 255 * 0.3 + Math.random() * 0.5;
        }

    }

    /**
    * 获得两点之间距离(2d x,y)
    * @param _firstpoint 
    * @param _secondpoint 
    */
    private getPointDistance(_firstpoint: Vec3, _secondpoint: Vec3) {
        let hordis: number = _secondpoint.x - _firstpoint.x;
        let verdis: number = _secondpoint.y - _firstpoint.y;
        return Math.sqrt(hordis * hordis + verdis * verdis);
    }

    /**
     * 获得两点之间角度
     * @param _firstpoint 
     * @param _secondpoint 
     */
    private getPointAngle(_firstpoint: Vec2, _secondpoint: Vec2) {
        return Math.atan2(_secondpoint.y - _firstpoint.y, _secondpoint.x - _firstpoint.x);
    }
}


